//
//  ProjectTableViewController.m
//  iTimesheet
//
//  Created by Mark Brindle on 06/04/2011.
//  Copyright 2011 ARKEMM Software Limited. All rights reserved.
//

#import "ProjectTableViewController.h"

// Added so we can get a reference to our App delegate
#import "iTimesheetAppDelegate.h"

// Added to get project details when populating cellForRowAtIndexPath
#import "PPProject.h"
#import "PPProject-TimeSpentText.h"

// Added to get layout used to populate our table with data for each cell
#import "ProjectTableViewCell.h"

// Added so we can navigate to a new view allowing user to enter new project details
#import "InsertProjectViewController.h"

// Added so we can navigate to job details for a selected project
#import "JobTableViewController.h"


@implementation ProjectTableViewController

// Synthesize code added for manually added properties
@synthesize projects;

#pragma mark - boilerplate code (provided by template)


- (id)initWithStyle:(UITableViewStyle)style
{
    self = [super initWithStyle:style];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)didReceiveMemoryWarning
{
    // Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
    
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void)viewDidUnload
{
    [super viewDidUnload];
    // Release any retained subviews of the main view.
    // e.g. self.myOutlet = nil;
}

- (void)viewDidAppear:(BOOL)animated
{
    [super viewDidAppear:animated];
}

- (void)viewWillDisappear:(BOOL)animated
{
    [super viewWillDisappear:animated];
}

- (void)viewDidDisappear:(BOOL)animated
{
    [super viewDidDisappear:animated];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
    return (interfaceOrientation == UIInterfaceOrientationPortrait);
}

#pragma mark - Table view data source

/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath
{
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath
{
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark - Existing methods (in the original template) that were amended


- (void)viewDidLoad
{
  [super viewDidLoad];
  // Added to set a title for the table view presented to the user
  [self setTitle:NSLocalizedString(@"Projects", @"Project names")];
  
  // Add an edit button to the left side of the navigation controller
  self.navigationItem.leftBarButtonItem = self.editButtonItem;
  
  // Add an 'Add' button (with the standard plus sign) on the right side of the
  // navigation controller
  UIBarButtonItem *addButton =
  [[UIBarButtonItem alloc] 
   initWithBarButtonSystemItem:UIBarButtonSystemItemAdd 
   target:self
   action:@selector(insertNewProject)];
  self.navigationItem.rightBarButtonItem = addButton;
  [addButton release];
  
  // Get a list of projects from core data via the application delegate
  iTimesheetAppDelegate *ad = [iTimesheetAppDelegate sharedAppDelegate];
  NSArray *list = [ad allInstancesOf:@"Project" orderedBy:@"name"];
  [self setProjects:[list mutableCopy]];

  // Uncomment the following line to preserve selection between presentations.
  // self.clearsSelectionOnViewWillAppear = NO;
  
  // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
  // self.navigationItem.rightBarButtonItem = self.editButtonItem;
}

- (void)dealloc
{
  // Added to release the memory allocated for our projects array
  [projects release];
  [super dealloc];
}

- (void)viewWillAppear:(BOOL)animated {
  [super viewWillAppear:animated];
  // Are we returning from InsertProjectViewController?
  if (pvc) {
    // Yes we are, so create a new project & add it to our array
    NSString *projectName = [pvc projectName];
    // Do we have a name?
    if ([projectName length] > 0) {
      // We have a name, so we can insert the new project via core data
      iTimesheetAppDelegate *ad = [iTimesheetAppDelegate sharedAppDelegate];
      NSManagedObjectContext *moc = [ad managedObjectContext];
      PPProject *project = [NSEntityDescription 
                            insertNewObjectForEntityForName:@"Project"
                            inManagedObjectContext:moc];
      // Now set some project details from data that the User provided
      [project setName:projectName];
      [project setDesc:[pvc projectDesc]];
      [project setCostCentre:[pvc projectCostCentre]];
      // And add this new project to our list of projects
      [projects addObject:project];
      // Re-sort the array into ascending alphabetical order of name
      NSSortDescriptor *sd = [[NSSortDescriptor alloc] initWithKey:@"name"
                                                         ascending:YES];
      NSArray *sds = [NSArray arrayWithObject:sd];
      [sd release];
      [projects sortUsingDescriptors:sds];
      // Re-display the table view, so our data shows up
      [[self tableView] reloadData];
    }
    // Release memory associated with our new project form & reset to nil so
    // that it will be recreated if required.
    [pvc release];
    pvc = nil;
  }
  [[self tableView] reloadData];
  // Clear the selection
  NSIndexPath *selectedPath = [[self tableView] indexPathForSelectedRow];
  if (selectedPath) {
    [[self tableView] deselectRowAtIndexPath:selectedPath animated:YES];
  }
}

// Override to support editing the table view.
-  (void)tableView:(UITableView *)tableView 
commitEditingStyle:(UITableViewCellEditingStyle)editingStyle 
 forRowAtIndexPath:(NSIndexPath *)indexPath {
  if (editingStyle == UITableViewCellEditingStyleDelete) {
    // Delete the project/managed object & also the row in the table
    [self removeProject:[indexPath row]];
    [tableView deleteRowsAtIndexPaths:[NSArray arrayWithObject:indexPath] withRowAnimation:UITableViewRowAnimationFade];
  }   
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
  // Return the number of sections.  In our case we only have one as we are
  // not grouping data in any way.
  return 1;
}

- (NSInteger)tableView:(UITableView *)tableView 
 numberOfRowsInSection:(NSInteger)section
{
  // Return the number of projects (that we retrieved from core data).
  return [[self projects] count];
}

- (UITableViewCell *)tableView:(UITableView *)tableView 
         cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
  // A unique descriptor for our table cell.  There's no significance to it;
  // it could be anything, so long as it's unique!
  static NSString *CellIdentifier = @"ProjectCellIdentifier";
  
  // Try and reuse an existing cell if available (it's more memory friendly).
  ProjectTableViewCell *cell = 
  (ProjectTableViewCell *)[tableView 
                           dequeueReusableCellWithIdentifier:CellIdentifier];
  // If we didn't get a cell to reuse create one from our ProjectTableViewCell,
  // which we need to load from our nib
  if (cell == nil) {
    NSArray *nib = [[NSBundle mainBundle] 
                    loadNibNamed:@"ProjectTableViewCell" 
                           owner:self 
                         options:nil];
  // Search through the nib and get the first object that conforms to our cell.
  // There should only be one, but as it's not an ordered list we go through
  // them one at a time until we find it.
    for (id oneObject in nib) {
      if ([oneObject isKindOfClass:[ProjectTableViewCell class]]) {
        cell = (ProjectTableViewCell *)oneObject;
        break;
      }
    }
  }
  
  // Now that we've got a cell, we want to put our project details into it for
  // display.  We use the table row as the index into our projects array so that
  // we get the correct project to present.
  PPProject *proj = [projects objectAtIndex:[indexPath row]];
  
  // Check that we got an object back - we should always have an object because
  // this method is called for each project that we said we had (via the 
  // delegate method numberOfRowsInSection, above).
  if (proj != nil) {
    // set the cell details from our project
    cell.nameField.text = [proj name];
    cell.timeField.text = [proj timeSpentText];
    cell.costCentreField.text = [NSString stringWithFormat:@"Cost centre: %@",
                                 [proj costCentre]];
    // Get the cost for jobs on this project
    cell.costField.text = [proj costText];
    cell.descField.text = [proj desc];

    // Set the disclosure button on the right hand side (allows user to drill-
    // down into jobs for this project).
    [cell setAccessoryType:UITableViewCellAccessoryDisclosureIndicator];
  }
  return cell;
}

// Table view delegate methods

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
  // This method is called when a cell is tapped.  
  // We are going to view job entries, so create a job table view controller
  // and push it onto the top of the view stack.
  JobTableViewController *vc = [[JobTableViewController alloc] init];
  [vc setProject:[projects objectAtIndex:[indexPath row]]];
  [[self navigationController] pushViewController:vc animated:YES];
  [vc release];
}

#pragma mark - Added methods (did not exist in original template code)

// Navigation Methods

-(void)insertNewProject {
  // This method is called when the user taps on the plus button at the right
  // side of the navigation menu to enter a new project.  Allocate & initialise
  // the insert view controller and push it onto the top of our navigation
  // controller's stack.
  pvc = [[InsertProjectViewController alloc] init];
  [[self navigationController] pushViewController:pvc animated:YES];
}

-(void) removeProject:(NSUInteger)index {
  // This method is called when the user wants to delete a row.  It is called 
  // via the delegate method .  Remove the managed object (via the managed 
  // object context) and also the reference in our projects array.
  PPProject *proj = [projects objectAtIndex:index];
  iTimesheetAppDelegate *ad = [iTimesheetAppDelegate sharedAppDelegate];
  NSManagedObjectContext *moc = [ad managedObjectContext];
  [moc deleteObject:proj];
  
  [projects removeObjectAtIndex:index];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
  // Delegate method called to determine the height required to display a cell.
  return kProjectTableRowHeight;
}

@end
